@@ -1,34 +1,22 @@
+"""Zenodo API for querying Zenodo."""
+
+from __future__ import annotations
+
 import json
 import os
+from abc import ABC
 from enum import Enum, unique
+from typing import Any, SupportsIndex
 
 import requests
+from requests import Response
+
+from album.core.utils.operations.url_operations import ResponseStatus
 
 
 class InvalidResponseStatusError(BaseException):
     """Raised when the response status is not the one expected."""
 
-    pass
-
-
-@unique
-class ResponseStatus(Enum):
-    """Response values and their name."""
-
-    OK = 200  # response included
-    Created = 201  # response included
-    Accepted = 202  # response included
-    NoContent = 204  # response NOT included
-    BadRequest = 400  # error response included
-    Unauthorized = 401  # error response included
-    Forbidden = 403  # error response included
-    NotFound = 404  # error response included
-    MethodNotAllowed = 405  # error response included
-    Conflict = 409  # error response included
-    UnsupportedMediaType = 415  # error response included
-    TooManyRequests = 429  # error response included
-    InternalServerError = 500  # error NOT response included
-
 
 @unique
 class DepositStatus(Enum):
@@ -96,7 +84,7 @@ class PublicationType(Enum):
     OTHER = "other"
 
 
-class ZenodoEntry(object):
+class ZenodoEntry(ABC):  # noqa: B024
     """Base class for all Zenodo API objects.
 
      The base class all Zenodo API objects inherit from.
@@ -109,8 +97,12 @@ class ZenodoEntry(object):
     """
 
     @staticmethod
-    def _get_attribute(entry_dict, key, required=False):
-        """Checks the dict for keys.
+    def _get_attribute(
+        entry_dict: dict[str, Any],
+        key: str,
+        required: bool = False,
+    ) -> Any:
+        """Check the dict for keys.
 
         Args:
             entry_dict:
@@ -132,8 +124,8 @@ class ZenodoEntry(object):
             raise AttributeError("Key %s not found but required" % key)
         return None
 
-    def __init__(self, entry_dict, base_url, access_token):
-        """Inits @ZenodoEntry.
+    def __init__(self, entry_dict: dict[str, Any], base_url: str, access_token: str):
+        """Init @ZenodoEntry.
 
         Args:
             entry_dict:
@@ -147,8 +139,8 @@ class ZenodoEntry(object):
         self.base_url = base_url
         self.params = {"access_token": access_token}
 
-    def to_dict(self):
-        """Removes sensitive information from the object and gives back its dictionary representation.
+    def to_dict(self) -> dict[str, Any]:
+        """Remove sensitive information from the object and gives back its dictionary representation.
 
         Returns:
             The dictionary ready for submission via API.
@@ -168,15 +160,14 @@ class ZenodoEntry(object):
         # treat special objects
         if "metadata" in d.keys():
             if isinstance(d["metadata"], ZenodoMetadata):
-                d["metadata"] = self.metadata.to_dict()
+                d["metadata"] = d["metadata"].to_dict()
 
         if "files" in d.keys():
-            if isinstance(d["files"], ZenodoFile):
-                d["files"] = [x.to_dict() for x in self.files]
+            d["files"] = [x.to_dict() for x in ZenodoDeposit.list_items(d["files"])]
 
         if "stats" in d.keys():
             if isinstance(d["stats"], ZenodoRecordStats):
-                d["stats"] = self.stats.to_dict()
+                d["stats"] = d["stats"].to_dict()
 
         return d
 
@@ -187,21 +178,22 @@ class ZenodoMetadata(ZenodoEntry):
     @classmethod
     def default_values(
         cls,
-        title,
-        creators,
-        description,
-        license,
-        version,
-        related_identifiers,
-        references,
+        title: str,
+        creators: list[str],
+        description: str,
+        license_: str,
+        version: str,
+        related_identifiers: str,
+        references: str,
     ):
+        """Get the default metadata object."""
         default_values = {
             "access_right": AccessRight.OPEN.value,
             "access_right_category": None,
             "creators": creators,
             "description": description,
             "doi": None,
-            "license": license,
+            "license": license_,
             "prereserve_doi": "true",
             "publication_date": None,
             "related_identifiers": related_identifiers,
@@ -214,7 +206,8 @@ class ZenodoMetadata(ZenodoEntry):
         }
         return cls(default_values)
 
-    def __init__(self, entry_dict):
+    def __init__(self, entry_dict: dict[str, Any]):
+        """Init the class."""
         super().__init__(entry_dict, "", "")
         self.access_right = self._get_attribute(entry_dict, "access_right")
         self.access_right_category = self._get_attribute(
@@ -240,49 +233,50 @@ class ZenodoMetadata(ZenodoEntry):
 class IterableList(list):
     """List for accessing objects in the list by a certain attribute or by the index."""
 
-    def __init__(self, id_attr):
+    def __init__(self, id_attr: str):
+        """Init the class."""
         super().__init__()
         self._id_attr = id_attr
 
-    def __contains__(self, attr):
+    def __contains__(self, attr: object):
+        """Check if an attribute is in the list."""
+        if not isinstance(attr, str):
+            return False
+
         try:
             getattr(self, attr)
             return True
         except (AttributeError, TypeError):
             return False
 
-    def __getattr__(self, attr):
+    def __getattr__(self, attr: str):
+        """Get an attribute."""
         for item in self:
             if getattr(item, self._id_attr) == attr:
                 return item
         return list.__getattribute__(self, attr)
 
-    def __getitem__(self, index):
-        if isinstance(index, int):
+    def __getitem__(self, index: SupportsIndex | slice | str) -> Any:
+        """Get item by index, slice, or id string."""
+        if isinstance(index, slice):
+            # delegate to list for normal slicing
             return list.__getitem__(self, index)
-
-        try:
-            return getattr(self, index)
-        except AttributeError as e:
-            raise IndexError("No item found with id %r" % index) from e
+        elif isinstance(index, int):
+            return list.__getitem__(self, index)
+        elif isinstance(index, str):
+            try:
+                return getattr(self, index)
+            except AttributeError as e:
+                raise IndexError(f"No item found with id {index!r}") from e
+        else:
+            raise TypeError(f"Invalid index type: {type(index).__name__}")
 
 
 class ZenodoFile(ZenodoEntry):
-    """Describes a file in a @ZenodoDeposit or a @ZenodoRecord."""
+    """Describe a file in a @ZenodoDeposit or a @ZenodoRecord."""
 
-    _id_attribute_ = "filename"
-
-    @classmethod
-    def list_items(cls, deposit):
-        out_list = IterableList(cls._id_attribute_)
-        out_list.extend(cls.iter_items(deposit))
-        return out_list
-
-    @classmethod
-    def iter_items(cls, deposit):
-        return (f for f in deposit._files)
-
-    def __init__(self, entry_dict):
+    def __init__(self, entry_dict: dict[str, Any]):
+        """Init the class."""
         super().__init__(entry_dict, "", "")
         self.checksum = self._get_attribute(entry_dict, "checksum")
         self.bucket = self._get_attribute(entry_dict, "bucket")
@@ -295,15 +289,30 @@ class ZenodoFile(ZenodoEntry):
         self.links = self._get_attribute(entry_dict, "links")
 
     # todo: write tests
-    def get_download_link(self):
+    def get_download_link(self) -> str:
+        """Get the download link of the file."""
         return self.links["self"]
 
 
 class ZenodoDeposit(ZenodoEntry):
     """The Zenodo Deposit class."""
 
-    def __init__(self, entry_dict, base_url, access_token):
-        """Inits the class.
+    _id_attribute_ = "filename"
+
+    @classmethod
+    def list_items(cls, deposit: ZenodoDeposit | ZenodoRecord):
+        """List all files of a deposit."""
+        out_list = IterableList(cls._id_attribute_)
+        out_list.extend(cls.iter_items(deposit))
+        return out_list
+
+    @classmethod
+    def iter_items(cls, deposit: ZenodoDeposit | ZenodoRecord):
+        """Iterate over the files of a deposit."""
+        return (f for f in deposit._files)
+
+    def __init__(self, entry_dict: dict[str, Any], base_url: str, access_token: str):
+        """Init the class.
 
         The dictionary usually comes from a API response. Files and metadata keys will have extra classes.
 
@@ -315,6 +324,10 @@ class ZenodoDeposit(ZenodoEntry):
             access_token:
                 The authentication token to use for querying the API.
         """
+        super().__init__(entry_dict, base_url, access_token)
+        self._init(entry_dict, base_url, access_token)
+
+    def _init(self, entry_dict: dict[str, Any], base_url: str, access_token: str):
         super().__init__(entry_dict, base_url, access_token)
         self.conceptdoi = self._get_attribute(entry_dict, "conceptdoi")
         self.conceptrecid = self._get_attribute(entry_dict, "conceptrecid")
@@ -344,25 +357,23 @@ class ZenodoDeposit(ZenodoEntry):
         self.files = files_init
 
     @property
-    def files(self):
-        return ZenodoFile.list_items(self)
+    def files(self) -> IterableList:
+        """Get the files of the deposit."""
+        return ZenodoDeposit.list_items(self)
 
     @files.setter
-    def files(self, files_init):
+    def files(self, files_init: IterableList | dict[str, Any]):
+        """Set the files of the deposit."""
+        files: IterableList = IterableList(self._id_attribute_)
         if files_init:
-            files = []
             for file_entry in files_init:
-                if isinstance(file_entry, ZenodoFile):
-                    files.append(file_entry)
-                else:
+                if isinstance(file_entry, dict):
                     files.append(ZenodoFile(file_entry))
-            self._files = files
-        else:
-            self._files = []
+        self._files = files
 
     # ############# Deposits attributes #############
 
-    def get_files_id_by_name(self, name):
+    def get_files_id_by_name(self, name: str) -> str | None:
         """Retrieve the id of a file given its name.
 
         Args:
@@ -390,7 +401,7 @@ class ZenodoDeposit(ZenodoEntry):
 
         return file_id
 
-    def list_files_name(self):
+    def list_files_name(self) -> list[str]:
         """Obtain a list of all filenames in the deposit.
 
         Returns:
@@ -404,8 +415,8 @@ class ZenodoDeposit(ZenodoEntry):
 
     # ############# Deposits operations #############
 
-    def reload(self):
-        """Reload the deposit. Resets all changes.
+    def reload(self) -> bool:
+        """Reload the deposit, thereby reset all changes.
 
         Returns:
              True if success.
@@ -420,12 +431,12 @@ class ZenodoDeposit(ZenodoEntry):
         response_dict = ZenodoAPI.validate_response(r, ResponseStatus.OK)
 
         # update object according to response
-        self.__init__(response_dict, self.base_url, self.params["access_token"])
+        self._init(response_dict, self.base_url, self.params["access_token"])
 
         return True
 
-    def update(self, zenodo_metadata):
-        """Updates the metadata of a deposit.
+    def update(self, zenodo_metadata: ZenodoMetadata | dict[str, Any]) -> bool:
+        """Update the metadata of a deposit.
 
         Args:
             zenodo_metadata:
@@ -442,6 +453,7 @@ class ZenodoDeposit(ZenodoEntry):
                 zenodo_metadata = ZenodoMetadata(zenodo_metadata)
             except AttributeError:
                 print("Please provide a valid dictionary as input!")
+                return False
 
         link = self.base_url + "/api/deposit/depositions/%s" % self.id
 
@@ -456,12 +468,12 @@ class ZenodoDeposit(ZenodoEntry):
         response_dict = ZenodoAPI.validate_response(r, ResponseStatus.OK)
 
         # update object according to response
-        self.__init__(response_dict, self.base_url, self.params["access_token"])
+        self._init(response_dict, self.base_url, self.params["access_token"])
 
         return True
 
-    def delete(self):
-        """Deletes an unpublished deposit.
+    def delete(self) -> bool:
+        """Delete an unpublished deposit.
 
         Returns:
              True if success.
@@ -469,7 +481,6 @@ class ZenodoDeposit(ZenodoEntry):
         Raises:
             InvalidResponseStatusError: If query response status other than expected.
         """
-
         # todo: check if not already published
 
         link = self.base_url + "/api/deposit/depositions/%s" % self.id
@@ -477,14 +488,14 @@ class ZenodoDeposit(ZenodoEntry):
         r = requests.delete(link, params=self.params)
 
         if ZenodoAPI.validate_response(r, ResponseStatus.NoContent):
-            self.__init__({}, "", "")
+            self._init({}, "", "")
 
         return True
 
     # ############# Deposits actions #############
 
-    def publish(self):
-        """Publishes an unpublished repo.
+    def publish(self) -> bool:
+        """Publish an unpublished repo.
 
         Returns:
              True if success.
@@ -492,7 +503,6 @@ class ZenodoDeposit(ZenodoEntry):
         Raises:
             InvalidResponseStatusError: If query response status other than expected.
         """
-
         # todo: check if already published
 
         link = self.base_url + "/api/deposit/depositions/%s/actions/publish" % self.id
@@ -501,12 +511,12 @@ class ZenodoDeposit(ZenodoEntry):
         response_dict = ZenodoAPI.validate_response(r, ResponseStatus.Accepted)
 
         # update object according to response
-        self.__init__(response_dict, self.base_url, self.params["access_token"])
+        self._init(response_dict, self.base_url, self.params["access_token"])
 
         return True
 
-    def new_version(self):
-        """Creates a new version of the deposit.
+    def new_version(self) -> list[ZenodoDeposit]:
+        """Create a new version of the deposit.
 
         Returns:
              True if success.
@@ -528,7 +538,7 @@ class ZenodoDeposit(ZenodoEntry):
         response_dict = ZenodoAPI.validate_response(r, ResponseStatus.OK)
 
         # update object according to response
-        self.__init__(response_dict, self.base_url, self.params["access_token"])
+        self._init(response_dict, self.base_url, self.params["access_token"])
 
         # return new version
         try:
@@ -545,8 +555,8 @@ class ZenodoDeposit(ZenodoEntry):
 
     # ############# Deposition files #############
 
-    def get_remote_files(self):
-        """Retrieve all files listed in a deposit
+    def get_remote_files(self) -> IterableList | None:
+        """Retrieve all files listed in a deposit.
 
         Returns:
              A list of @ZenodoFile objects.
@@ -560,16 +570,12 @@ class ZenodoDeposit(ZenodoEntry):
 
         response_dict = ZenodoAPI.validate_response(r, ResponseStatus.OK)
 
-        file_list = []
-        for file_dict in response_dict:
-            file_list.append(ZenodoFile(file_dict))
+        self.files = response_dict  # type: ignore[assignment]
 
-        self.files = file_list if file_list else None
+        return self.files
 
-        return file_list
-
-    def create_file(self, file):
-        """Uploads a new file to the deposit.
+    def create_file(self, file) -> ZenodoFile:
+        """Upload a new file to the deposit.
 
         Args:
             file:
@@ -594,8 +600,8 @@ class ZenodoDeposit(ZenodoEntry):
 
         return ZenodoFile(response_dict)
 
-    def delete_file_by_id(self, file_id):
-        """Deletes the file with a certain ID in the deposit.
+    def delete_file_by_id(self, file_id: str) -> bool:
+        """Delete the file with a certain ID in the deposit.
 
         Args:
             file_id:
@@ -607,9 +613,9 @@ class ZenodoDeposit(ZenodoEntry):
         Raises:
             InvalidResponseStatusError: If query response status other than expected.
         """
-        link = self.base_url + "/api/deposit/depositions/%s/files/%s" % (
-            self.id,
-            file_id,
+        link = self.base_url + "/api/deposit/depositions/{id}/files/{file_id}".format(
+            id=self.id,
+            file_id=file_id,
         )
 
         r = requests.delete(link, params=self.params)
@@ -620,8 +626,8 @@ class ZenodoDeposit(ZenodoEntry):
 
         return True
 
-    def update_file_by_id(self, file_id, new_file):
-        """Updates a existent file with the new version.
+    def update_file_by_id(self, file_id: str, new_file: str) -> ZenodoFile:
+        """Update an existent file with the new version.
 
         Args:
             file_id:
@@ -638,8 +644,8 @@ class ZenodoDeposit(ZenodoEntry):
         self.delete_file_by_id(file_id)
         return self.create_file(new_file)
 
-    def delete_file(self, file_name):
-        """Deletes a file in a deposit given its name
+    def delete_file(self, file_name: str) -> bool:
+        """Delete a file in a deposit given its name.
 
         Args:
             file_name:
@@ -653,10 +659,12 @@ class ZenodoDeposit(ZenodoEntry):
         """
         file_id = self.get_files_id_by_name(file_name)
         if file_id:
-            self.delete_file_by_id(file_id)
+            return self.delete_file_by_id(file_id)
 
-    def update_file(self, file_name, new_file):
-        """Updates a file in a deposit given its name
+        return False
+
+    def update_file(self, file_name: str, new_file: str) -> ZenodoFile | None:
+        """Update a file in a deposit given its name.
 
         Args:
             file_name:
@@ -672,28 +680,32 @@ class ZenodoDeposit(ZenodoEntry):
         """
         file_id = self.get_files_id_by_name(file_name)
         if file_id:
-            self.update_file_by_id(file_id, new_file)
+            return self.update_file_by_id(file_id, new_file)
+
+        return None
 
 
 class ZenodoRecord(ZenodoDeposit):
     """Class for the @ZenodoRecord. Holds the statistics (@ZenodoRecordStats) of a published deposit."""
 
-    def __init__(self, entry_dict, base_url, access_token):
+    def __init__(self, entry_dict: dict[str, Any], base_url: str, access_token: str):
+        """Init the class."""
         super().__init__(entry_dict, base_url, access_token)
         self.owners = self._get_attribute(entry_dict, "owners")
         self.revision = self._get_attribute(entry_dict, "revision")
         self.stats = ZenodoRecordStats(self._get_attribute(entry_dict, "stats"))
         self.updated = self._get_attribute(entry_dict, "updated")
 
-    def print_stats(self):
-        """Prints the dictionary."""
+    def print_stats(self) -> None:
+        """Print the dictionary."""
         print(self.stats.to_dict())
 
 
 class ZenodoRecordStats(ZenodoEntry):
     """Class holding the statistics of a @ZenodoRecord."""
 
-    def __init__(self, entry_dict):
+    def __init__(self, entry_dict: dict[str, Any]):
+        """Init the class."""
         super().__init__(entry_dict, "", "")
         self.downloads = self._get_attribute(entry_dict, "downloads")
         self.unique_downloads = self._get_attribute(entry_dict, "unique_downloads")
@@ -712,17 +724,21 @@ class ZenodoRecordStats(ZenodoEntry):
 
 
 class ZenodoDefaultUrl(Enum):
+    """Default URLs for Zenodo."""
+
     sandbox_url = "https://sandbox.zenodo.org/"
     url = "https://zenodo.org/"
 
 
 class ZenodoAPI:
-    """
-    Zenodo API. Querying the ZenodoAPI.
-    """
+    """Zenodo API for querying the ZenodoAPI."""
 
-    def __init__(self, base_url=ZenodoDefaultUrl.url.value, access_token=None):
-        """Inits the object given a base_url and an access_token.
+    def __init__(
+        self,
+        base_url: str = ZenodoDefaultUrl.url.value,
+        access_token: str | None = None,
+    ):
+        """Init the object given a base_url and an access_token.
 
         Args:
             base_url:
@@ -737,8 +753,10 @@ class ZenodoAPI:
             self.params["access_token"] = access_token
 
     @staticmethod
-    def validate_response(response, expected_response_code=None):
-        """Validates the response from a request.
+    def validate_response(
+        response: Response, expected_response_code: ResponseStatus | None = None
+    ) -> dict[str, Any]:
+        """Validate the response from a request.
 
         Args:
             response:
@@ -747,7 +765,7 @@ class ZenodoAPI:
                 The expected response specified in the API
 
         Returns:
-            JSON response body or "{'true'}" if no body included in response.
+            JSON response body or '{"": True}' if no body included in response.
 
         Raises:
             InvalidResponseStatusError: If query response status other than expected.
@@ -774,14 +792,16 @@ class ZenodoAPI:
                     "Expected %s got %s"
                     % (expected_response_code.name, status_code.name)
                 )
-            return json.dumps(True)
+            return {"": True}
         else:
             print("Error: %s" % status_code.name)
+            if status_code == ResponseStatus.Forbidden:
+                print("Forbidden operation. Is your access token valid?")
             if status_code != ResponseStatus.InternalServerError:
-                print("Detailed message:")
-                print(response.json()["message"])
-                if response.json()["errors"]:
-                    print(response.json()["errors"])
+                json_response = response.json()
+                print("Detailed message: %s" % json_response["message"])
+                if hasattr(json_response, "errors"):
+                    print("Errors: %s" % json_response["errors"])
 
         raise InvalidResponseStatusError(
             "Error '%s' occurred. See Log for detailed information!" % status_code.name
@@ -791,11 +811,11 @@ class ZenodoAPI:
 
     def deposit_get(
         self,
-        deposit_id=None,
-        q="",
-        status=DepositStatus.PUBLISHED,
-        sort=SortOrder.BEST_MATCH,
-    ):
+        deposit_id: str | None = None,
+        q: str = "",
+        status: DepositStatus = DepositStatus.PUBLISHED,
+        sort: SortOrder = SortOrder.BEST_MATCH,
+    ) -> list[ZenodoDeposit]:
         """Retrieve a deposit.
 
         Args:
@@ -811,7 +831,6 @@ class ZenodoAPI:
         Returns:
             A list of @ZenodoDeposit found. Empty if none found.
         """
-
         link = self.base_url + "/api/deposit/depositions"
 
         if deposit_id:
@@ -842,13 +861,12 @@ class ZenodoAPI:
 
         return deposit_list
 
-    def deposit_create(self):
-        """Creates an empty deposit (already uploaded).
+    def deposit_create(self) -> ZenodoDeposit:
+        """Create an empty deposit (already uploaded).
 
         Returns:
             A ZenodoDeposit object.
         """
-
         link = self.base_url + "/api/deposit/depositions"
 
         r = requests.post(link, params=self.params, json={})
@@ -857,7 +875,18 @@ class ZenodoAPI:
 
         return ZenodoDeposit(response_dict, self.base_url, self.params["access_token"])
 
-    def deposit_create_with_prereserve_doi(self, metadata):
+    def deposit_create_with_prereserve_doi(
+        self, metadata: ZenodoMetadata | dict[str, Any]
+    ) -> ZenodoDeposit:
+        """Create a deposit with pre-reserved DOI.
+
+        Args:
+            metadata:
+                The metadata of the deposit. Either as dictionary or as @ZenodoMetadata object.
+
+        Returns:
+            A ZenodoDeposit object.
+        """
         deposit = self.deposit_create()
         deposit.update(metadata)
 
@@ -868,12 +897,12 @@ class ZenodoAPI:
     # todo: write tests
     def records_get(
         self,
-        record_id=None,
-        q="",
-        status=DepositStatus.PUBLISHED,
-        sort=SortOrder.BEST_MATCH,
-        record_type=UploadType.SOFTWARE,
-    ):
+        record_id: str = "",
+        q: str = "",
+        status: DepositStatus = DepositStatus.PUBLISHED,
+        sort: SortOrder = SortOrder.BEST_MATCH,
+        record_type: UploadType = UploadType.SOFTWARE,
+    ) -> list[ZenodoRecord]:
         """Retrieve a record or searches through published records.
 
         Args:
@@ -891,7 +920,6 @@ class ZenodoAPI:
         Returns:
             Returns the @ZenodoRecord object.
         """
-
         link = self.base_url + "/api/records"
 
         if record_id:
