@@ -1,24 +1,15 @@
+"""Interface for the state manager."""
 from abc import ABCMeta, abstractmethod
-from typing import Optional
 
 from album.runner.core.api.model.solution import ISolution
 
 
 class IStateManager:
+    """Interface for the state manager."""
+
     __metaclass__ = ABCMeta
 
     @abstractmethod
-    def load(self, path) -> Optional[ISolution]:
-        raise NotImplementedError
-
-    @abstractmethod
-    def pop_active_solution(self):
-        raise NotImplementedError
-
-    @abstractmethod
-    def get_active_solution(self) -> Optional[ISolution]:
-        raise NotImplementedError
-
-    @abstractmethod
-    def get_parent_solution(self) -> Optional[ISolution]:
+    def load(self, path) -> ISolution:
+        """Load the state from the given path."""
         raise NotImplementedError
