@@ -1,22 +1,29 @@
+"""Module for git operations."""
+
 import os
 import re
 from contextlib import contextmanager
 from pathlib import Path
-from typing import Generator
+from typing import Generator, List, Union
 from urllib.parse import urlparse
 
 import git
-from git import Repo
+from git import Head, Repo
+from git.refs import HEAD
 
-from album.core.utils.operations.file_operations import force_remove, create_path_recursively, folder_empty
+from album.core.utils.operations.file_operations import (
+    create_path_recursively,
+    folder_empty,
+    force_remove,
+)
 from album.core.utils.operations.url_operations import is_url
 from album.runner import album_logging
 
 module_logger = album_logging.get_active_logger
 
 
-def checkout_branch(git_repo, branch_name):
-    """Checks out a branch on a repository.
+def checkout_branch(git_repo, branch_name) -> HEAD:
+    """Check out a branch on a repository.
 
     First, local refs are taken, then refs pointing to origin.
 
@@ -35,7 +42,8 @@ def checkout_branch(git_repo, branch_name):
     """
     try:  # checkout branch locally
         module_logger().debug(
-            "Found the following branches locally: \n %s..." % "\n".join([h.name for h in git_repo.heads])
+            "Found the following branches locally: \n %s..."
+            % "\n".join([h.name for h in git_repo.heads])
         )
         head = git_repo.heads[branch_name]
         head.checkout()
@@ -55,11 +63,11 @@ def checkout_branch(git_repo, branch_name):
             except IndexError:
                 module_logger().debug("Not found! Trying next...")
         module_logger().debug("Nothing left to try...")
-        raise IndexError("Branch \"%s\" not in repository!" % branch_name) from e
+        raise IndexError('Branch "%s" not in repository!' % branch_name) from e
 
 
 def retrieve_files_from_head(head, pattern, number_of_files=1, relative=False):
-    """Extracts a file with a pattern given a branch (or head) of a repository.
+    """Extract a file with a pattern given a branch (or head) of a repository.
 
     Args:
         number_of_files:
@@ -80,33 +88,35 @@ def retrieve_files_from_head(head, pattern, number_of_files=1, relative=False):
     pattern_matches = head.repo.git.ls_files(pattern)
 
     res = []
-    for file in pattern_matches.split('\n'):
+    for file in pattern_matches.split("\n"):
         # unfortunately git on windows internally uses linux-separator as path separator.
         # Paths therefore might have the wrong separator.
-        file = file.replace('/', os.path.sep)
+        file = file.replace("/", os.path.sep)
         module_logger().debug("Found file: %s..." % file)
         if relative:
             res.append(file)
         else:
             res.append(os.path.join(head.repo.working_tree_dir, file))
 
-
     if not res:
-        raise RuntimeError("Head \"%s\" does not hold pattern \"%s\"! Aborting..." % (head.name, pattern))
+        raise RuntimeError(
+            f'Head "{head.name}" does not hold pattern "{pattern}"! Aborting...'
+        )
 
     if number_of_files > 0:
         if len(res) != number_of_files:
             raise RuntimeError(
-                "Head \"%s\" holds pattern \"%s\" %s times, but expected %s. Aborting..." %
-                (head.name, pattern, len(res), number_of_files)
+                'Head "%s" holds pattern "%s" %s times, but expected %s. Aborting...'
+                % (head.name, pattern, len(res), number_of_files)
             )
 
     return res
 
 
 def retrieve_files_from_head_last_commit(head, pattern, option="", number_of_files=1):
-    """Extracts a file with a "startswith" pattern given a branch (or head) of a repository, only factoring in
-    files that were changed in the last commit.
+    """Extract a file with a "startswith" pattern given a branch (or head) of a repository.
+
+    That only factor in files that were changed in the last commit.
 
     Args:
         number_of_files:
@@ -142,47 +152,65 @@ def retrieve_files_from_head_last_commit(head, pattern, option="", number_of_fil
     for file in diff:
         # unfortunately git on windows internally uses linux-separator as path separator.
         # Paths therefore might have the wrong separator.
-        path = file.b_path.replace('/', os.path.sep)
+        path = file.b_path.replace("/", os.path.sep)
         module_logger().debug("Found file in commit diff to parent: %s..." % path)
         module_logger().debug("Matching pattern %s..." % pattern)
 
         if option == "startswith":
             if path.startswith(pattern):
-                module_logger().debug("Found file matching pattern %s: %s..." % (pattern, path))
-                abs_path_solution_file.append(os.path.join(head.repo.working_tree_dir, path))
+                module_logger().debug(
+                    f"Found file matching pattern {pattern}: {path}..."
+                )
+                abs_path_solution_file.append(
+                    os.path.join(head.repo.working_tree_dir, path)
+                )
         else:
             if re.search(pattern, path):
-                module_logger().debug("Found file matching pattern %s: %s..." % (pattern, path))
-                abs_path_solution_file.append(os.path.join(head.repo.working_tree_dir, path))
+                module_logger().debug(
+                    f"Found file matching pattern {pattern}: {path}..."
+                )
+                abs_path_solution_file.append(
+                    os.path.join(head.repo.working_tree_dir, path)
+                )
 
     if not abs_path_solution_file:
-        raise RuntimeError("Head \"%s\" does not hold pattern \"%s\"! Aborting..." % (head.name, pattern))
+        raise RuntimeError(
+            f'Head "{head.name}" does not hold pattern "{pattern}"! Aborting...'
+        )
 
     if number_of_files > 0:
         if len(abs_path_solution_file) != number_of_files:
             raise RuntimeError(
-                "Head \"%s\" holds pattern \"%s\" %s times, but expected %s. Aborting..." %
-                (head.name, pattern, len(abs_path_solution_file), number_of_files)
+                'Head "%s" holds pattern "%s" %s times, but expected %s. Aborting...'
+                % (head.name, pattern, len(abs_path_solution_file), number_of_files)
             )
 
     return abs_path_solution_file
 
 
-def _add_files(repo, file_paths) -> bool:
+def _add_files(repo: Repo, file_paths: List[Path]) -> bool:
     """Add files to the repo in the branch currently checked out."""
     if repo.index.diff(None) or repo.untracked_files:
-        module_logger().info('Preparing committing...')
+        module_logger().info("Preparing committing...")
         for file_path in file_paths:
-            module_logger().debug('Adding file %s...' % file_path)
+            module_logger().debug("Adding file %s..." % file_path)
             # todo: nice catching here?
             repo.git.add(file_path)
         return True
     return False
 
 
-def add_files_commit_and_push(head, file_paths, commit_message, push=False, email=None, username=None,
-                              push_option_list=None, force=False):
-    """Adds files in a given path to a git head and commits.
+def add_files_commit_and_push(
+    head: Union[Head, HEAD],
+    file_paths: List[Path],
+    commit_message: str,
+    push: bool = False,
+    email: str = "",
+    username: str = "",
+    push_option_list: Union[List[str], None] = None,
+    force: bool = False,
+) -> None:
+    """Add files in a given path to a git head and commits.
 
     Args:
         push_option_list:
@@ -207,9 +235,9 @@ def add_files_commit_and_push(head, file_paths, commit_message, push=False, emai
 
     """
     if push_option_list is None or push_option_list == []:
-        push_options = []
+        push_options_ = []
     else:
-        push_options = ["-o %s" % o for o in push_option_list]
+        push_options_ = ["-o %s" % o for o in push_option_list]
 
     repo = head.repo
 
@@ -219,30 +247,32 @@ def add_files_commit_and_push(head, file_paths, commit_message, push=False, emai
     if _add_files(repo, file_paths):
 
         # build command
-        cmd_option = ['--set-upstream']
-        cmd = cmd_option + push_options
+        cmd_option = ["--set-upstream"]
+        cmd = cmd_option + push_options_
         if force:
-            cmd = cmd + ['-f']
+            cmd = cmd + ["-f"]
 
         remote_name = get_remote_name(repo)
 
         cmd = cmd + [remote_name, head]
 
-        module_logger().debug("Running command: repo.git.push(%s)..." % (", ".join(str(x) for x in cmd)))
+        module_logger().debug(
+            "Running command: repo.git.push(%s)..." % (", ".join(str(x) for x in cmd))
+        )
 
         # commit
         repo.git.commit(m=commit_message)
 
         if push:
-            module_logger().info('Preparing pushing...')
+            module_logger().info("Preparing pushing...")
             repo.git.push(cmd)
 
     else:
         raise RuntimeError("Diff shows no changes to the repository. Aborting...")
 
 
-def add_tag(repo, tag):
-    """Adds a tag to the most recent commit.
+def add_tag(repo: Repo, tag: str) -> None:
+    """Add a tag to the most recent commit.
 
     Args:
         repo:
@@ -250,12 +280,13 @@ def add_tag(repo, tag):
         tag:
             The tag associated with the commit
     """
-    repo.git.tag('-a', tag, '-f', '-m', '')
+    repo.git.tag("-a", tag, "-f", "-m", "")
     remote_name = get_remote_name(repo)
-    repo.git.push([remote_name, tag, '-f'])
+    repo.git.push([remote_name, tag, "-f"])
 
 
-def get_remote_name(repo):
+def get_remote_name(repo: Repo) -> str:
+    """Get the name of the remote repository."""
     try:
         remote_name = repo.remote().refs.HEAD.remote_name
     except AttributeError:
@@ -263,8 +294,8 @@ def get_remote_name(repo):
     return remote_name
 
 
-def remove_files(head, file_paths):
-    """Adds files in a given path to a git head and commits.
+def remove_files(head: Head, file_paths: List[str]) -> None:
+    """Add files in a given path to a git head and commits.
 
     Args:
         head:
@@ -276,12 +307,12 @@ def remove_files(head, file_paths):
 
     if repo.index.diff(None) or repo.untracked_files:
         for file_path in file_paths:
-            module_logger().debug('Removing file %s...' % file_path)
-            repo.git.rm([file_path, '-r', '--ignore-unmatch', '--cached'])
+            module_logger().debug("Removing file %s..." % file_path)
+            repo.git.rm([file_path, "-r", "--ignore-unmatch", "--cached"])
 
 
-def remove_tag(repo, tag):
-    """Adds files in a given path to a git head and commits.
+def remove_tag(repo: Repo, tag: str) -> None:
+    """Add files in a given path to a git head and commits.
 
     Args:
         repo:
@@ -290,25 +321,29 @@ def remove_tag(repo, tag):
             The tag to remove
     """
     remote_name = get_remote_name(repo)
-    repo.git.tag('-d', tag)
-    repo.git.push('--delete', remote_name, tag)
+    repo.git.tag("-d", tag)
+    repo.git.push("--delete", remote_name, tag)
 
 
-def get_tags(repo):
-    tags = [tag.name for tag in sorted(repo.tags, key=lambda t: t.commit.committed_datetime)]
+def get_tags(repo: Repo) -> List[str]:
+    """Get all tags of a repository."""
+    tags = [
+        tag.name for tag in sorted(repo.tags, key=lambda t: t.commit.committed_datetime)
+    ]
     tags.reverse()
     return tags
 
 
-def revert(repo, tag, files: list):
+def revert(repo: Repo, tag: str, files: List[str]):
+    """Revert files to a given tag."""
     for file in files:
-        repo.git.reset(tag, '--', file)
-        repo.git.checkout('--', file)
-        repo.git.clean('-fd', file)
+        repo.git.reset(tag, "--", file)
+        repo.git.checkout("--", file)
+        repo.git.clean("-fd", file)
 
 
-def configure_git(repo, email, username):
-    """Configures email and username to use for git operations for the given repo.
+def configure_git(repo: Repo, email: str, username: str) -> Repo:
+    """Configure email and username to use for git operations for the given repo.
 
     Args:
         repo:
@@ -324,10 +359,10 @@ def configure_git(repo, email, username):
 
     """
     if username:
-        module_logger().info("Set username to \"%s\"" % username)
+        module_logger().info('Set username to "%s"' % username)
         repo.config_writer().set_value("user", "name", username).release()
     if email:
-        module_logger().info("Set email to \"%s\"" % email)
+        module_logger().info('Set email to "%s"' % email)
         repo.config_writer().set_value("user", "email", email).release()
     if not username and not email:
         module_logger().warning("Neither username nor email given! Doing nothing...")
@@ -336,9 +371,8 @@ def configure_git(repo, email, username):
 
 
 # todo: write test
-def create_new_head(repo, name):
-    """Force creates a new head of a given name in a repository and returns the head."""
-
+def create_new_head(repo: Repo, name: str) -> Head:
+    """Force create a new head of a given name in a repository and returns the head."""
     if name in repo.heads:
         git.Head.delete(repo, name, force=True)
     new_head = repo.create_head(name)
@@ -346,8 +380,13 @@ def create_new_head(repo, name):
     return new_head
 
 
-def download_repository(repo_url, git_folder_path, force_download=True, update=True) -> Repo:
-    """Downloads or updates the repository behind a url, returns repository object on success.
+def download_repository(
+    repo_url: str,
+    git_folder_path: Union[str, Path],
+    force_download: bool = True,
+    update: bool = True,
+) -> Repo:
+    """Download or update the repository behind a url, returns repository object on success.
 
     If repository already cached, head is detached to origin HEAD for a clean start for new branches.
 
@@ -381,21 +420,26 @@ def download_repository(repo_url, git_folder_path, force_download=True, update=T
         if force_download:
             force_remove(git_folder_path)
 
-        module_logger().info("Download repository from %s in %s..." % (repo_url, git_folder_path))
+        module_logger().info(
+            f"Download repository from {repo_url} in {git_folder_path}..."
+        )
         repo = git.Repo.clone_from(repo_url, git_folder_path)
 
     return repo
 
 
-def checkout_files(repo, files_to_download):
+def checkout_files(repo: Repo, files_to_download: List[str]) -> None:
+    """Checkout files in a repository."""
     for file in files_to_download:
         repo.git.restore(file, staged=True)
         repo.git.checkout(file)
 
 
 @contextmanager
-def clone_repository_sparse(repo_url, branch_name, target_repo_path) -> Generator[Repo, None, None]:
-    """Clones a repository branch to a given path.
+def clone_repository_sparse(
+    repo_url: str, branch_name: str, target_repo_path: Union[str, Path]
+) -> Generator[Repo, None, None]:
+    """Clone a repository branch to a given path.
 
     Args:
         repo_url:
@@ -410,17 +454,28 @@ def clone_repository_sparse(repo_url, branch_name, target_repo_path) -> Generato
     force_remove(git_folder_path)
     create_path_recursively(git_folder_path)
 
-    module_logger().debug("Cloning repository without history from %s into %s..." % (repo_url, git_folder_path))
+    module_logger().debug(
+        "Cloning repository without history from %s into %s..."
+        % (repo_url, git_folder_path)
+    )
     repo = git.Repo.clone_from(
-        repo_url, git_folder_path, branch=branch_name, no_checkout=True, depth=1, no_tags=True, single_branch=True
+        repo_url,
+        git_folder_path,
+        branch=branch_name,
+        no_checkout=True,
+        depth=1,
+        no_tags=True,
+        single_branch=True,
     )
     yield repo
     repo.close()
 
 
 @contextmanager
-def clone_repository(src, target_repo_path, force=False) -> Generator[Repo, None, None]:
-    """Clones the full repository
+def clone_repository(
+    src: Path, target_repo_path: Union[Path, str], force: bool = False
+) -> Generator[Repo, None, None]:
+    """Clone the full repository.
 
     Args:
         src:
@@ -430,13 +485,12 @@ def clone_repository(src, target_repo_path, force=False) -> Generator[Repo, None
         force:
             boolean value. If true deletes target folder first
 
-
     """
     if not folder_empty(target_repo_path):
         if force:
             force_remove(target_repo_path)
         else:
-            raise RuntimeError("Target folder \"%s\" not empty!" % str(target_repo_path))
+            raise RuntimeError('Target folder "%s" not empty!' % str(target_repo_path))
 
     create_path_recursively(target_repo_path)
     repo = git.Repo.clone_from(src, target_repo_path)
@@ -445,8 +499,10 @@ def clone_repository(src, target_repo_path, force=False) -> Generator[Repo, None
     repo.close()
 
 
-def init_repository(path):
-    """Initializes a repository to the origin reference. Thereby discarding all changes made to the repository.
+def init_repository(path: Union[Path, str]) -> Repo:
+    """Initialize a repository to the origin reference.
+
+    Thereby discarding all changes made to the repository.
 
     Usually this means checking out the origin "main" branch, but this depends on the repository configuration.
 
@@ -477,55 +533,64 @@ def init_repository(path):
     return repo
 
 
-def create_bare_repository(target):
+def create_bare_repository(target: Path) -> None:
+    """Create a new bare repository in the given path."""
     create_path_recursively(target)
 
     repo = git.Repo.init(target, bare=True)
 
     # ref HEAD to main
-    repo.git.symbolic_ref('HEAD', 'refs/heads/main')
+    repo.git.symbolic_ref("HEAD", "refs/heads/main")
 
     repo.close()
 
 
-def create_repository(target):
+def create_repository(target: str) -> Repo:
+    """Create a new repository in the given path."""
     create_path_recursively(target)
 
     repo = git.Repo.init(target)
 
     # ref HEAD to main
-    repo.git.symbolic_ref('HEAD', 'refs/heads/main')
+    repo.git.symbolic_ref("HEAD", "refs/heads/main")
 
     return repo
 
 
-def clean_repository(repo, target_head_name=None):
-    """Resets all changes made in the current repository"""
+def clean_repository(repo: Repo, target_head_name: str = "") -> None:
+    """Reset all changes made in the current repository."""
     if not target_head_name:
         target_head_name = get_local_remote_ref_head(repo).name
     remote_name = repo.remote().name
     remote_ref = remote_name + "/" + target_head_name
     # reset to current remote HEAD reference name
-    repo.git.reset(['--hard', remote_ref])
+    repo.git.reset(["--hard", remote_ref])
     # remove all leftover-untracked files (if any)
-    repo.git.clean('-fd')
+    repo.git.clean("-fd")
 
 
-def get_local_remote_ref_head(repo):
+def get_local_remote_ref_head(repo: Repo) -> Head:
+    """Get the local remote reference head of the repository."""
     if repo.remote().refs:
         try:
             remote_head = repo.git.remote(["set-head", "origin", "-a"])
             remote_main_name = remote_head.split(" ")[-1]
         except git.GitCommandError:
             remote_main_name = "main"
+        # Sanitize the name, as it might contain ' or " when it is "main"
+        # E.g. when the branch is called "main",  remote_main_name="'origin/HEAD' is unchanged and points to 'main'"
+        remote_main_name = remote_main_name.strip("'\"")
         head = repo.heads[remote_main_name]
     else:
         head = repo.head.ref
     return head
 
 
-def checkout_main(repo, main_name=None):
-    """Checks out the main branch of the repository locally. Note: must not be called "main"!"""
+def checkout_main(repo: Repo, main_name: str = "") -> Head:
+    """Check out the main branch of the repository locally.
+
+    Note: must not be called "main"!
+    """
     if not main_name:
         head = get_local_remote_ref_head(repo)
     else:
@@ -539,8 +604,8 @@ def checkout_main(repo, main_name=None):
     return head
 
 
-def retrieve_default_mr_push_options(repo_url) -> list:
-    """Returns the default push option for the given repository host if available (e.g. gitlab, github)
+def retrieve_default_mr_push_options(repo_url: str) -> List[str]:
+    """Return the default push option for the given repository host if available (e.g. gitlab, github).
 
     Args:
         repo_url:
